@@ -34,7 +34,7 @@ impl World {
     /// Do nothing if component has been registered.
     pub fn register<T : Component>(&mut self) -> &mut Self {
         let type_id = TypeId::of::<T>();
-        if self.components.contains_key(&type_id) {
+        if self.has_registered::<T>() {
             return self;
         }
         self.components.insert(
@@ -47,6 +47,12 @@ impl World {
         self
     }
 
+    /// Check if a component has been registered
+    pub fn has_registered<T : Component>(&self) -> bool {
+        let type_id = TypeId::of::<T>();
+        self.components.contains_key(&type_id)
+    }
+
     /// Create an empty entity in world, return an EntityRef.
     pub fn create_entity(&mut self) -> EntityRef<'_>{
         let id = self.entity_manager.create();
@@ -59,7 +65,9 @@ impl World {
         //remove all components of this entity
         for (_,storage) in &mut self.components{
             let mut storage = storage.borrow_mut();
-            storage.remove(entity_id);
+            if storage.has(entity_id) {
+                storage.remove(entity_id);
+            }
         }
     }
 
